#! /usr/bin/perl
#
# Common functions for the NoCat module
#
# Author: Sven Backhausen <backhausen@lanthan.de>
#

do '../web-lib.pl';
&init_config();

# Load OS-specific functions
if (-r "$gconfig{'os_type'}-$gconfig{'os_version'}-lib.pl") {
	        do "$gconfig{'os_type'}-$gconfig{'os_version'}-lib.pl";
} else {
		do "$gconfig{'os_type'}-lib.pl";
}

# get_nocat_support()
# Returns 0 for no support, 1 for gateway only, 2 for authserv, 3 for gateway & authserv
# FIXME Does anybody know a better way to detect an authserv?
sub get_nocat_support {
	my $nocat_ability = 0;
	if ( -e $config{'gateway_conf'} && -x $config{'gateway_cmd'} ) {
		$nocat_ability++;
		}
	if ( -e $config{'authserv_conf'} ) {
		$nocat_ability += 2;
		}
	return $nocat_ability;
}


# returns a hash containing the default values of the configuration
# directives from authservdefaults.in and gatewaydefaults.in
# expects a service name (gateway|authserv) as argument.
sub readdefaults {
        my %defaults;
        my $filename = $_[0] . "defaults.in";
        open DEFAULTS, "$filename"
                or die "Cannot open default definitions file. ($!)";
        while (<DEFAULTS>) {
                chomp;
                my @default = split /\s+/, $_;
                my $key = shift @default;
                my $value = join " ", @default;
                $defaults{$key} = $value ;
        }
        close DEFAULTS;
        %defaults;
}


# returns an array containing the available directives
# expects a service name (gateway|authserv) as argument.
sub readdirectives {
	my @directives;
	my $filename = $_[0] . "defaults.in";	#read from the defaults file
        open DIRECTIVES, "$filename"
                or die "Cannot open directive definitions file. ($!)";
        while (<DIRECTIVES>) {
                chomp;
		unless (m/^\s*#/) {		#ignore comments
			/^\s*\b(\w+)\b/;
			$_ = $1;
			push(@directives, $_);
		}
        }
        close DIRECTIVES;
        @directives;
}


# returns a hash containing the directive as key and the value as value.
# expects a complete filename as argument 0
# expects a service name (gateway|authserv) as argument 1
sub readcurrentconfig {
	my %config;
	my $filename = $_[0];
	my $service = $_[1];
	my @directives = &readdirectives($service);
	my @founddirectives;
	open CONFIG, $filename
		or die "Cannot open config file. ($!)";
	while (<CONFIG>) {
                chomp;
  		if (/^\s*[@directives]/) {
			# found a directive. Save the directive name.
			/^\s*\b(\w+)\b/;
			my $directive = $1;
			push(@founddirectives, $directive);
			# Remove the directive name and all leading
               		# whitespaces and you get the value and comments (if any).
			$_ =~ s/^\s*$directive\s*//;
               		# remove the comments (if any)
               		$_ =~ s/#.*//;
			$config{$directive} = $_;
			delete $directives[indexof($directive, @directives)];
               	}
	}
	foreach (@directives){
		# set the value of %config{$directive_def} to 1 and %config{$directive} to ""
		if (/\w+/){ # delete leaves some empty values in array.
			$config{$_ . _def} = 1;
			$config{$_} = "";
		}
        }
        close CONFIG;
        %config;
}


# returns a hash containing the index of the field identifiers in the Members table (nocat DBI)
# FIXME This is a hardcoded static mapping. It would be better to generate it dynamically from
# FIXME the values in the authserv.conf, but they are incomplete in the default (sample) config.
sub get_database_fielddef {
	my %fields;
	$fields{User}=5;
	$fields{Name}=7;
	$fields{Pass}=6;
	$fields{Status}=4;
	$fields{Modified}=3;
	$fields{Created}=2;
	$fields{Url}=0;
	$fields{Description}=1;
	%fields;
}

# returns "started" if Gateway is running or "stopped" if Gateway is stopped.
sub get_gateway_status {
	my $status;
	if ( &find_byname('$config{gateway_cmd}') ) {
		$status = "started";
	} else {
		$status = "stopped";
	}
	$status;
}
